/****************************************************************
 *								*
 *	Douglas - Peucker Line Simplification Algorithm.	*
 *								*
 ****************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <errno.h>

#define MAXPOINTS 10000

struct dppoint {
    float x;
    float y;
} pts[MAXPOINTS], simp[MAXPOINTS];

int n, simpptr;

float e;  /*    . */

void dpbase (int i, int j)
{
    float maxdist = 0, tmpdist;
    int k, f;
    
    for (k = i+1; k < j; k++)
    {
	/*     k-    ij. */
	tmpdist = (pts[j].x*pts[k].y + pts[i].x*pts[j].y + pts[k].x*pts[i].y - pts[j].x*pts[i].y - pts[i].x*pts[k].y - pts[k].x*pts[j].y) / sqrt ((pts[i].x-pts[j].x)*(pts[i].x-pts[j].x) + (pts[i].y-pts[j].y)*(pts[i].y-pts[j].y));
	
	if (tmpdist < 0) tmpdist = -tmpdist;  /*   . */
	
	if (tmpdist > maxdist)   /*      k-   */
	{			 /*        ,    */
	    maxdist = tmpdist;	 /*        . */
	    f = k;
	}
    }
    
    if (maxdist > e)	/*         */
    {			/*    ,   -  */
	dpbase (i, f);	/*     ,     */
	dpbase (f, j);	/*             */
    }			/*     .  			     */
    else
	simp[simpptr++] = pts[i];  /*   -   i- . */
}


void readfile (FILE *f)
{    
    /*  ,     . */
    while (!feof (f))
    {
	simpptr = 0;
	
	/*     n- . */
	fscanf (f, "%f %f\n", &pts[n].x, &pts[n].y);
	
	/*       -. */
	dpbase (0, n);	
	
	if (n > 0)
	    simp[simpptr] = pts[n];

	n++;
    }
}


void writefile (FILE *f)
{
    int i;
    
    /*     . */
    for (i = 0; i <= simpptr; i++)
	fprintf (f,"%f %f\n", simp[i].x, simp[i].y);
}


int main (int argc, char *argv[])
{
    FILE *fin, *fout; /*    . */

    /*          ,
	  ,    
	-    . */
    if (argc != 4)
    {
	printf ("Usage: %s in_file out_file dist\n", argv[0]);
	exit (-1);
    }
    
    /*      . */
    if ((fin = fopen (argv[1], "r")) == NULL)
    {
	printf ("%s: %s: %s\n", argv[0], argv[1], strerror (errno));
	exit (-1);
    }
    
    /*      . */
    if ((fout = fopen (argv[2], "w")) == NULL)
    {
	printf ("%s: %s: %s\n", argv[0], argv[2], strerror (errno));
	fclose (fin);
	exit (-1);
    }

    /*    . */
    e = atof (argv[3]);
    
    n = 0;
    
    /*        
	 -. */
    readfile (fin);
    
    /*       . */
    writefile (fout);
    
    /*    . */
    fclose (fin);
    fclose (fout);
    
    exit (0);
}
