/*
 * File: test.c
 * Author: Andrew Salo
 * Date: 16.03.03
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
     
#define TESTS 50
#define TESTPATH "tests/"
#define DP_ALG "./dp"
#define DP_EPS "3"
#define SIMP_ALG "./simple"


/*   */
int test_dp (const char *);
int test_simp (const char *);
int write_table (const char *);


/*
 * int main (int, char**) ---
 *  
 */
int main (int argc, char **argv)
{
  int i, ln;
  char test[30];

  strcpy (test, TESTPATH "test\0");
  ln = strlen (test);

  for (i = 0; i < TESTS; i++)
    {
      /*   */
      sprintf (&test[ln], "%d\0", i);

      /*   - */
      if (test_dp (test) == -1)
	{
	  perror ("test_dp");
	  exit (1);
	}

      /*    */
      if (test_simp (test) == -1)
	{
	  perror ("test_simp");
	  exit (1);
	}

      /*   */
      if (write_table (test) == -1)
	exit (1);

      /*    */
      getchar ();
    }

  exit (0);
}


/*
 * int test_dp (const char*) ---
 *   -
 */
int test_dp (const char* test)
{
  int status;
  pid_t pid;

  char test_out[30];

  /*    */
  strcpy (test_out, test);
  strcat (test_out, ".dp\0");

  switch (pid = fork ())
    {
    case -1:
      /*  */
      return -1;
      break;

    case 0:
      /*  */
      execlp (DP_ALG, DP_ALG, test, test_out, DP_EPS, NULL);
      _exit (1);
      break;

    default:
      /*  */
      if (waitpid (pid, &status, 0) != pid)
	return -1;

      break;
    }

  return 0;
}


/*
 * int test_simp (const char*) ---
 *   
 */
int test_simp (const char* test)
{
  int status;
  pid_t pid;

  char test_out[30];

  /*    */
  strcpy (test_out, test);
  strcat (test_out, ".simp\0");

  switch (pid = fork ())
    {
    case -1:
      /*  */
      return -1;
      break;

    case 0:
      /*  */
      execlp (SIMP_ALG, SIMP_ALG, test, test_out, NULL);
      _exit (1);
      break;

    default:
      /*  */
      if (waitpid (pid, &status, 0) != pid)
	return -1;

      break;
    }

  return 0;
}


/*
 * write_table (const char *) ---
 *    
 */
int write_table (const char* test)
{
  FILE *in_pts, *dp_pts, *simp_pts;
  char test_in[30], test_dp[30], test_simp[30];
  int in_flag = 0, dp_flag = 0, simp_flag = 0;
  float x, y;

  /*   */
  strcpy (test_in, test);
  strcat (test_in, ".in\0");

  strcpy (test_dp, test);
  strcat (test_dp, ".dp\0");

  strcpy (test_simp, test);
  strcat (test_simp, ".simp\0");

  /*   */
  if ((in_pts = fopen (test_in, "r")) == NULL)
    {
      perror (test_in);
      return -1;
    }

  if ((dp_pts = fopen (test_dp, "r")) == NULL)
    {
      perror (test_dp);
      fclose (in_pts);
      return -1;
    }

  if ((simp_pts = fopen (test_simp, "r")) == NULL)
    {
      perror (test_simp);
      fclose (in_pts);
      fclose (dp_pts);
      return -1;
    }

  /*  */
  fprintf (stdout, "\nTest: %s\n", test);

  fprintf (stdout, "--------------------------+-------------"
	   "-------------+--------------------------\n"
	   "      Initial points      |"
	   "      Douglas-Peucker     |"
	   "         Modified         \n"
	   "--------------------------+-------------"
	   "-------------+--------------------------\n");
  /*     */
  while ((in_flag * dp_flag * simp_flag) != 1)
    {
      /*     */
      if (!in_flag)
	{
	  fscanf (in_pts, "%f %f", &x, &y);

	  if (feof (in_pts))
	    {
	      in_flag = 1;
	      fprintf (stdout, "\t\t\t  |");
	    }
	  else
	    fprintf (stdout, "  %10.3f  %10.3f  |", x, y);
	}
      else
	fprintf (stdout, "\t\t\t  |");

      /*   ,   
	  -  */
      if (!dp_flag)
	{
	  fscanf (dp_pts, "%f %f", &x, &y);

	  if (feof (dp_pts))
	    {
	      dp_flag = 1;
	      fprintf (stdout, "\t\t\t     |");
	    }
	  else
	    fprintf (stdout, "  %10.3f  %10.3f  |", x, y);
	}
      else
	fprintf (stdout, "\t\t\t     |");

      /*   ,  
	   */
      if (!simp_flag)
	{
	  fscanf (simp_pts, "%f %f", &x, &y);

	  if (feof (simp_pts))
	    {
	      simp_flag = 1;
	      fprintf (stdout, "\t\t\t  \n");
	    }
	  else
	    fprintf (stdout, "  %10.3f  %10.3f  \n", x, y);
	}
      else
	fprintf (stdout, "\t\t\t  \n");
    }

  fprintf (stdout, "--------------------------+-------------"
	   "-------------+--------------------------\n");

  /*   */
  fclose (in_pts);
  fclose (dp_pts);
  fclose (simp_pts);

  return 0;
}
